#ifndef CONFIGWINDOW_H_INCLUDED_
#define CONFIGWINDOW_H_INCLUDED_

#include <iostream>
#include <sstream>
#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <glm/gtx/transform.hpp>

#include "../fluid_project/CFluidWindow.h"
#include "../../../common/Wavefront.h"
#include "Model.h"

class  ConfigWindow : public CFluidWindow {
    GLfloat* pthetaY;
    GLfloat* pthetaX;
    GLfloat* ppositionZ;
    GLfloat* pscale;
    Model** ppmodel;
    char const* pfilepath; 

public:
    ConfigWindow(GLfloat* pthetaY, GLfloat* pthetaX, GLfloat* ppositionZ, GLfloat* pscale, Model** ppmodel, char const* pfilepath)
        : pthetaY(pthetaY),
          pthetaX(pthetaX),
          ppositionZ(ppositionZ),
          pscale(pscale),
          ppmodel(ppmodel),
          pfilepath(pfilepath)
    {
    }

    void show()
    {
        {
            std::ostringstream ostr;
            ostr << *pthetaY;
            txt_thetaY->value( ostr.str().c_str() );
            slider_thetaY->value( *pthetaY );
        }

        {
            std::ostringstream ostr;
            ostr << *pthetaX;
            txt_thetaX->value( ostr.str().c_str() );
            slider_thetaX->value( *pthetaX );
        }

        {
            std::ostringstream ostr;
            ostr << *ppositionZ;
            txt_positionZ->value( ostr.str().c_str() );
            slider_positionZ->value( *ppositionZ );
        }

        {
            std::ostringstream ostr;
            ostr << *pscale;
            txt_scale->value( ostr.str().c_str() );
            slider_scale->value( *pscale );
        }

        if ( pfilepath ) {
            txt_file_path->value( pfilepath );
        }

        slider_thetaY->callback(callback_slider_thetaY, (void*)this);
        slider_thetaX->callback(callback_slider_thetaX, (void*)this);
        slider_positionZ->callback(callback_slider_positionZ, (void*)this);
        slider_scale->callback(callback_slider_scale, (void*)this);

        btn_update->callback((Fl_Callback*)callback_btn_update, (void*)(this));    
        btn_load_file->callback((Fl_Callback*)callback_btn_load_file, (void*)(this));    
        pwindow->callback( (Fl_Callback*)callback_window_closing, (void*)(this) );

        pwindow->show();
    }

    void slider_update_thetaY()
    {
        static int recurse = 0;
        if ( recurse ) { 
            return;
        } else {
            recurse = 1;
            std::string tmp;
            std::ostringstream ostr;
            ostr << slider_thetaY->value();
            txt_thetaY->value(ostr.str().c_str());          // pass slider's value to input
            recurse = 0;
            click_btn_update();
        }
    }

    void slider_update_thetaX()
    {
        static int recurse = 0;
        if ( recurse ) { 
            return;
        } else {
            recurse = 1;
            std::string tmp;
            std::ostringstream ostr;
            ostr << slider_thetaX->value();
            txt_thetaX->value(ostr.str().c_str());          // pass slider's value to input
            recurse = 0;
            click_btn_update();
        }
    }

    void slider_update_positionZ()
    {
        static int recurse = 0;
        if ( recurse ) { 
            return;
        } else {
            recurse = 1;
            std::string tmp;
            std::ostringstream ostr;
            ostr << slider_positionZ->value();
            txt_positionZ->value(ostr.str().c_str());          // pass slider's value to input
            recurse = 0;
            click_btn_update();
        }
    }

    void slider_update_scale()
    {
        static int recurse = 0;
        if ( recurse ) { 
            return;
        } else {
            recurse = 1;
            std::string tmp;
            std::ostringstream ostr;
            ostr << slider_scale->value();
            txt_scale->value(ostr.str().c_str());          // pass slider's value to input
            recurse = 0;
            click_btn_update();
        }
    }

    void click_btn_update()
    {
        {
            std::istringstream istr( txt_thetaY->value() );
            GLfloat tmp;
            if ( istr >> tmp ) {
                *pthetaY = glm::radians( tmp );
            } else {
                txt_thetaY->value( "**BAD**" );
            }
        }
        {
            std::istringstream istr( txt_thetaX->value() );
            GLfloat tmp;
            if ( istr >> tmp ) {
                *pthetaX = glm::radians( tmp );
            } else {
                txt_thetaX->value( "**BAD**" );
            }
        }
        {
            std::istringstream istr( txt_positionZ->value() );
            GLfloat tmp;
            if ( istr >> tmp ) {
                *ppositionZ = tmp;
            } else {
                txt_positionZ->value( "**BAD**" );
            }
        }
        {
            std::istringstream istr( txt_scale->value() );
            GLfloat tmp;
            if ( istr >> tmp ) {
                *pscale = tmp;
            } else {
                txt_positionZ->value( "**BAD**" );
            }
        }
    }

    void click_btn_load_file()
    {
        Model* p_old_model = *ppmodel; // pointer to old model

        WavefrontObjLoader loader( txt_file_path->value() );
        if ( !loader.load() ) {
            txt_file_path->value( "*** FAILED ***");
            return;
        }

        // Create and load the new model
        assert( loader.num_vertices );

        *ppmodel = NULL;    // set world's pointer to model to NULL to disable rendering

        Model* myModel = new Model(p_old_model->shader);

        myModel->set_geometry( loader.vertices, loader.num_vertices * 3 * sizeof( GLfloat ) );
        if ( loader.num_normals ) { myModel->set_normals( loader.normals, loader.num_normals * 3 * sizeof( GLfloat ) ); }
        if ( loader.num_uvs ) { myModel->set_texture_coordinates( loader.UVs, loader.num_uvs * 2 * sizeof( GLfloat ) ); }
        if ( loader.num_indices ) { myModel->set_index_buffer( loader.indices, loader.num_indices * sizeof( GLuint ) ); }

        GLfloat material_ambient[] = {0.7f, 0.0f, 0.0f, 1.0f};
        GLfloat material_diffuse[] = {0.7f, 0.1f, 0.1f, 1.0f};

        GLfloat material_specular[] = {1.0f, 1.0f, 1.0f, 1.0f};
        GLfloat shininess = 100.0f;

        //myModel->set_material(p_old_model->combined_ambient, p_old_model->combined_diffuse, p_old_model->combined_specular, p_old_model->shininess);
        myModel->set_material(material_ambient, material_diffuse, material_specular, p_old_model->shininess);
        myModel->set_light(p_old_model->light);
        myModel->upload_2_server();

        delete p_old_model; // destroy the old model
        *ppmodel = myModel; // start rendering
    }

    // FLTK callback functions:
    static void callback_slider_thetaY(Fl_Widget *widg, void *userdata_) {
        ((ConfigWindow*)userdata_)->slider_update_thetaY();
    }

    static void callback_slider_thetaX(Fl_Widget *widg, void *userdata_) {
        ((ConfigWindow*)userdata_)->slider_update_thetaX();
    }

    static void callback_slider_positionZ(Fl_Widget *widg, void *userdata_) {
        ((ConfigWindow*)userdata_)->slider_update_positionZ();
    }

    static void callback_slider_scale(Fl_Widget *widg, void *userdata_) {
        ((ConfigWindow*)userdata_)->slider_update_scale();
    }

    static void callback_btn_update(Fl_Widget* widg, void* userdata_) {
        //std::cout << "button update clicked\n";
        ConfigWindow* config_window = static_cast< ConfigWindow* > (userdata_);
        config_window->click_btn_update();
    }

    static void callback_btn_load_file(Fl_Widget* widg, void* userdata_) {
        ConfigWindow* config_window = static_cast< ConfigWindow* > (userdata_);
        config_window->click_btn_load_file();
    }

    static void callback_window_closing( Fl_Widget* widg, void* userdata_ ) {
        std::cout << "X button clicked\n";
        while (Fl::first_window()) {
            Fl::first_window()->hide();
        }
    }
};

#endif // CONFIGWINDOW_H_INCLUDED_