#include <GL/glew.h>
#include <GL/freeglut.h> 
#include "ConfigWindow.h" // FLTK window support
#include "Shader.h"
#include "Model.h"
#include "FPSCamera.h"

#include "Light.h"                // UPDATE!
#include "../../../common/Wavefront.h"

// c262_lab07_x2.cpp
// shader path configuration:
#define    BCC_PROJECT_NAME "c262_lab07_x2"
#define    BCC_ROOT_PATH "../../labs/"
#define    BCC_VERTEX_SHADER BCC_ROOT_PATH BCC_PROJECT_NAME "/Shaders/vertexProgram_v.c"
#define    BCC_FRAGMENT_SHADER  BCC_ROOT_PATH BCC_PROJECT_NAME "/Shaders/fragmentProgram_f.c"

//------------------------------------------------------------------
// HOW TO TEST DIFFERENT MODELS:
//------------------------------------------------------------------
// Include one of the models defined in a header file,
// -or-
// load te model from the OBJ file by defining MODEL_OBJ_FILENAME
//------------------------------------------------------------------

//#include "../../../common/Objects/phlegm_small.h"    // UPDATE! - look at this file if you haven't!  
                            // It has the vertices[], normals[] and indices[] in it

//#include "../../../common/Objects/dragon.h"
//#include "../../../common/Objects/teapot.h" // use scale_amount = 10.0f;
//#include "../../../common/Objects/cube.h" // scale_amount = 0.5f;

//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/bunny-1500.obj"     // scale_amount = 100.0f;
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/bunny-5000.obj"    // scale_amount = 100.0f;
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/cow-1500.obj"      //scale_amount = 40.0f;
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/cow-5000.obj"   //scale_amount = 40.0f;
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/cup.obj"          // scale_amount = 0.01f;
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/dragon-1500.obj"  // scale_amount = 40.0f;
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/dragon-5000.obj"    // scale_amount = 2.0f
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/lamp600.obj"        // scale_amount = 0.007f
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/lego.obj"            // scale_amount = 0.001f
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/shammy.obj"          // scale_amount = 0.001f
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/snakie.obj"          // scale_amount = 0.001f
//#define MODEL_OBJ_FILENAME "../../Models/Spheretree/teapot.obj"      // scale_amount = 0.001f

//#define MODEL_OBJ_FILENAME "../../Models/Misc/tetrahedron.obj.txt" // crashes the driver
//#define MODEL_OBJ_FILENAME "../../Models/Misc/cube5.obj" // a bigger cube (5 units) -- scale_amount = 0.15f;
//#define MODEL_OBJ_FILENAME "../../Models/Misc/cube_blender.obj.txt" // works fine
//#define MODEL_OBJ_FILENAME "../../Models/Misc/dragon.obj" // scale_amount = 40.0f;
//#define MODEL_OBJ_FILENAME "../../Models/Misc/teapot.obj" // from http://people.sc.fsu.edu/~jburkardt/data/obj/obj.html
//#define MODEL_OBJ_FILENAME "../../Models/Misc/car.obj" // scale_amount = 3.0f;
//#define MODEL_OBJ_FILENAME "../../Models/Lab8/OBJ/cube.obj" // scale_amount = 9.0f;
#define MODEL_OBJ_FILENAME "../../Models/Lab8/OBJ/teapot.obj" // scale_amount = 9.0f;

Model* myModel;
FPSCamera* camera;
Light*    light;                    // UPDATE!

GLfloat  thetaX = 0.0f;
GLfloat  thetaY = 0.0f;
GLfloat  positionZ = -50.0f;
GLfloat  scale_amount;
GLfloat fov = 60.0f;
GLfloat aspect = 1.0f;

void animate()
{
    Fl::wait(0.0); // FLTK window support
    glutPostRedisplay();
}

void render() {
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    if ( myModel ) {
        glm::mat4 mtx_scale = glm::scale(
            glm::mat4( 1.0f ),
            glm::vec3( scale_amount )
            );

        myModel->mR = glm::rotate(
            glm::rotate(
                glm::mat4( 1.0f ),
                thetaX, //glm::degrees( thetaX ),
                glm::vec3(1.0f, 0.0f, 0.0f)
                ),
            thetaY, //glm::degrees( thetaY ),
            glm::vec3(0.0f, 1.0f, 0.0f)
            );

        glm::mat4 mtx_trans = glm::translate(
            mtx_scale,
            glm::vec3( 0.0f, 0.0f, positionZ )
            );

        // Compute the model matrix:
        // The order here is important!
        //myModel->mM =  mtx_trans * mtx_scale * myModel->mR; // rotate first
        myModel->mM =  mtx_trans * myModel->mR * mtx_scale;   // scale first
        // Compute the matrix for rotation of normals:
        myModel->mR = glm::transpose( glm::inverse( myModel->mM ));

        // We'll only set the View and Perspective matrix when/if we change shaders
        camera->set_uniform_view_perspective(myModel->shader->program_ID);
        light->set_shader_light_position();                            // UPDATE!
        myModel->render();
    }

    glutSwapBuffers();
}

// Any time the window is resized, this function gets called. Set by the
// "glutReshapeFunc" in main.
void change_viewport(int w, int h){
    aspect = GLfloat( w ) / h;
    camera->set_perspective( fov, aspect, 1.0f, 1000.0f );
    glViewport(0, 0, w, h);
}

int main (int argc, char** argv) {
    // Initialize GLUT
    glutInit(&argc, argv);

    // Set up some memory buffers for our display
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_MULTISAMPLE );
    // Set the window size
    glutInitWindowSize(800, 800);
    // Create the window with the title "Triangle"
    glutCreateWindow("Week 4. OpenGL Vertex Array and Vertex Buffer Objects");
    // Bind the two functions (above) to respond when necessary
    glutReshapeFunc(change_viewport);
    glutDisplayFunc(render);
    glutIdleFunc( animate );
    
    // Remember, we always init GLEW after we create a window
    // This asks the GPU driver for all of those gl functions
    if (GLEW_OK != glewInit()) {
        exit(1);
    }
    glClearColor(0.63f, 0.75f, 0.95f, 1.0f);
    glEnable (GL_DEPTH_TEST);
    glEnable( GL_MULTISAMPLE );
    //glPolygonMode( GL_FRONT_AND_BACK, GL_POINT );
    //glLineWidth( 2.0f );

    Shader* myShader = new Shader( BCC_VERTEX_SHADER, BCC_FRAGMENT_SHADER );
    
    //=========================  UPDATE!  =====================
    // Directional light parameters:
    GLfloat light_pos[] = {0.0f, 2.0f, 10.0f, 1.0f/*0.0f==Direction light, 1.0f otherwise*/};        

    GLfloat light_ambient[] = {0.0f, 0.2f, 0.0f, 1.0f};
    GLfloat light_diffuse[] = {1.0f, 1.0f, 0.0f, 1.0f};
    GLfloat light_specular[] = {1.0f, 1.0f, 1.0f, 1.0f};
    /*ANSWER*/
    ////GLfloat light_diffuse[] = {0.0f, 0.6f, 0.2f, 1.0f};
    ////GLfloat light_specular[] = {0.0f, 0.8f, 0.7f, 1.0f};

    GLfloat material_ambient[] = {0.0f, 0.4f, 0.0f, 1.0f};
    GLfloat material_diffuse[] = {0.0f, 0.4f, 0.0f, 1.0f};
    GLfloat material_specular[] = {0.0f, 1.0f, 0.0f, 1.0f};
    /*ANSWER*/
    ////GLfloat material_ambient[] = {0.0f, 0.7f, 0.0f, 1.0f};
    ////GLfloat material_diffuse[] = {0.1f, 0.7f, 0.1f, 1.0f};

    GLfloat shininess = 150.0f;

    light = new Light(light_pos, light_ambient, light_diffuse, light_specular, myShader);
    //========================================================
    
    myModel = new Model(myShader);

#ifdef MODEL_OBJ_FILENAME
    WavefrontObjLoader loader( MODEL_OBJ_FILENAME );
    if ( !loader.load() ) {
        std::cout << "Failed to load OBJ file, exiting!" << std::endl;
        return 1;
    }
    assert( loader.num_vertices );
    myModel->set_geometry( loader.vertices, loader.num_vertices * 3 * sizeof( GLfloat ) );
    if ( loader.num_normals ) { myModel->set_normals( loader.normals, loader.num_normals * 3 * sizeof( GLfloat ) ); }
    if ( loader.num_uvs ) { myModel->set_texture_coordinates( loader.UVs, loader.num_uvs * 2 * sizeof( GLfloat ) ); }
    if ( loader.num_indices ) { myModel->set_index_buffer( loader.indices, loader.num_indices * sizeof( GLuint ) ); }
#else
    myModel->set_geometry( vertices, num_vertices * 3 * sizeof( GLfloat ) );
    if ( num_normals ) { myModel->set_normals( normals, num_normals * 3 * sizeof( GLfloat ) ); }
    //if ( num_uvs ) { myModel->set_texture_coordinates( UVs, num_uvs * 2 * sizeof( GLfloat ) ); }
    if ( num_indices ) { myModel->set_index_buffer( indices, num_indices * sizeof( GLuint ) ); }
#endif

    myModel->set_material(material_ambient, material_diffuse, material_specular, shininess);        // UPDATE!
    myModel->set_light(light);                                                    // UPDATE!
    myModel->upload_2_server();

    // Set up the camera 
    camera = new FPSCamera(fov, aspect, 1.0f, 1000.0f);
    // Set up some matrices for the model
    scale_amount = 9.0f;
    //scale_amount = 10.0f;
    //scale_amount = 100.0f;
    //scale_amount = 40.0f;
    //scale_amount = 0.01f;
    //scale_amount = 0.15f;
    //scale_amount = 3.0f;

    // FLTK window support
    ConfigWindow config_window( &thetaY, &thetaX, &positionZ, &scale_amount, &myModel, MODEL_OBJ_FILENAME );
    config_window.show();
    glutMainLoop();

    return 0;
}
